#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-12 15:43:06
# Filename      : config.py
# Description   : 
from __future__ import print_function, unicode_literals

from vacuum import datatype
import os
import sys

RC_FILE = os.path.join(os.path.expanduser('~'), '.vacuumrc')

RC_DEFAULT_CONTENT = b"""{
    'warning_rules'     :       
    [
        {
            'name'      :   'timeout',
            'intervals' :   60,
            'threshold' :   20,
            }, # 在10秒内出现3次timeout相关的异常，则预警
        {
            'name'      :   'assert',
            'intervals' :   30,
            'threshold' :   4,
            },
        {
            'name'      :   'default',
            'intervals' :   30,
            'threshold' :   5,
            }, # default表示默认情况
        ],
    'qiniu'     :   {
        'ak'    :   '',
        'sk'    :   '',
        'bucket':   'vacuum',
        },
    'submail'   :   {
        'appid' :   '',
        'appkey':   '',
        'sign_type':    'normal',
        'sender':   'warning@tuxpy.org',
        'to'    :   ['q8886888@qq.com'],
        },
    'ucpaas'    :   {
        'appid' :   '',
        'sid'   :   '',
        'token' :   '',
        'to'    :   ['18989502070'],
        'template_id': '22633',
        },
    }
"""

def _init_rc():
    print('initialize rc')
    with open(RC_FILE, 'wb') as fd:
        fd.write(RC_DEFAULT_CONTENT)

    print('Please complement', RC_FILE)
    sys.exit(1)

def _get_config_from_rc():
    if not os.path.exists(RC_FILE):
        _init_rc()

    with open(RC_FILE, 'rb') as fd:
        content = b''
        for line in fd:
            content += line[:line.find(b'#')]

    return eval(content)

def _get_log_path():
    return os.environ.get('DSB_WARNING_LOG', '/var/log/vacuum')

def _get_warning_rules():
    """返回预警规则
    [{
        'name'              :       异常名称，会进行正则匹配,
        'threshold'         :       异常数的阈值,
        'intervals'         :       有效异常数据的间隔时间,
        }]
    """
    return _get_config_from_rc()['warning_rules']

def get_config():
    config = datatype.ObjectDict(_get_config_from_rc())
    config.update(datatype.ObjectDict({
        'log_path'      :       _get_log_path(),
        'warning_rules' :       _get_warning_rules(),
        }))

    return config




import hashlib
import urllib2
import urllib

class Mail:
    '''
    @init
    '''
    def __init__(self, mail_configs):
        self.sign_type = 'normal'
        self.mail_configs = mail_configs
        self.signature = ''
    
    '''
    @createSignature
    '''
    def __create_signature(self, request):
        if self.sign_type == 'normal':
            self.signature = self.mail_configs['appkey']
        else:
            self.__build_signature(request)

    '''
    @buildSignature
    '''
    def __build_signature(self, request):
        appid = self.mail_configs['appid']
        appkey = self.mail_configs['appkey']
        para_keys = request.keys();
        para_keys.sort()
        sign_str = ''
        for key in para_keys:
            if key.find('attachments') == -1:
                sign_str += "%s=%s&"%(key,request[key])
        sign_str = appid+appkey+sign_str[:-1]+appid+appkey
        if self.sign_type == 'md5':
            hash=hashlib.md5()
            hash.update(sign_str)
            self.signature = hash.hexdigest()
        elif self.sign_type == 'sha1':
            hash=hashlib.sha1() 
            hash.update(sign_str)
            self.signature = hash.hexdigest()

    def __http_get(self,url):
        return eval(urllib2.urlopen(url = url).read())

    def __http_post(self, url, para):
        data = urllib.urlencode(para) 
        return eval(urllib2.urlopen(url=url,data = data).read())
         
    '''
    @getTimestamp
    '''
    def get_timestamp(self):
        api = 'https://api.submail.cn/service/timestamp.json'
        resp = self.__http_get(api)
        return resp['timestamp']

    '''
    @Send
    '''
    def send(self,request):
        '''
        @setup API httpRequest URI
        '''
        api = 'https://api.submail.cn/mail/send.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.mail_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()

        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if self.mail_configs.has_key('sign_type') == False:
            self.sign_type = 'normal'
        elif self.mail_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.mail_configs['sign_type'] 
            request['sign_type'] = self.mail_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature

        '''
        create final API post query End
        '''

        #print request
        '''
        @send request
        '''
        return self.__http_post(api, request)
        
    '''
    @xsend
    '''
    def xsend(self, request):
        '''
        @setup API httpRequest URI
        '''
        api = 'https://api.submail.cn/mail/xsend.json'


        '''
        create final API post query Start
        '''
        request['appid'] = self.mail_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()

        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if self.mail_configs.has_key('sign_type') == False:
            self.sign_type = 'normal'
        elif self.mail_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.mail_configs['sign_type'] 
            request['sign_type'] = self.mail_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature

        '''
        create final API post query End
        '''

        '''
        @send request
        '''
        return self.__http_post(api, request)

    '''
    addressbook/mail/subscribe
    '''
    def subscribe(self, request):
        '''
        @setup API httpRequest URI
        '''
        api='https://api.submail.cn/addressbook/mail/subscribe.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.mail_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()
        
        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if self.mail_configs.has_key('sign_type') == False:
            self.sign_type = 'normal'
        elif self.mail_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.mail_configs['sign_type'] 
            request['sign_type'] = self.mail_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature

        '''
        '''

        '''
        @subscribe request
        '''
        return self.__http_post(api, request)

    '''
    addressbook/mail/unsubscribe
    '''
    def unsubscribe(self, request):
        '''
        @setup API httpRequest URI
        '''
        api='https://api.submail.cn/addressbook/mail/unsubscribe.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.mail_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()
        
        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if self.mail_configs.has_key('sign_type') == False:
            self.sign_type = 'normal'
        elif self.mail_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.mail_configs['sign_type'] 
            request['sign_type'] = self.mail_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature

        '''
        create final API post query End
        '''

        '''
        @unsubscribe request
        '''
        return self.__http_post(api, request)



'''
if __name__ == '__main__':
       mail_obj = Mail()
       print mail_obj.get_timestamp()
'''
